library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
library unisim;
use unisim.vcomponents.all;

entity bscan is
Port(
    result:in std_logic_vector(2 downto 0);
    random_number_i:out std_logic_vector(31 downto 0)
    );
end bscan;

architecture Behavioral of bscan is
signal bscan_tck_c_a           : std_logic;
signal bscan_tdo_c_a           : std_logic;
signal bscan_sel_c_a           : std_logic;
signal bscan_shift_c_a         : std_logic;
signal bscan_capture_c_a       : std_logic;
signal bscan_update_c_a        : std_logic;

signal status_reg :std_logic_vector(31 downto 0):=(others=>'0');
signal bscan_tdo: std_logic;
----------------------------------------------------
signal bscan_tck_c_b           : std_logic;
signal bscan_sel_c_b           : std_logic;
signal bscan_shift_c_b         : std_logic;
signal bscan_capture_c_b       : std_logic;
signal bscan_update_c_b        : std_logic;
signal bscan_tdi: std_logic;

signal shift_reg_in          : std_logic_vector(31 downto 0);
signal read_reg             : std_logic_vector(31 downto 0):=(others=>'0');

begin
random_number_i<=read_reg;


BSCAN_A: BSCANE2
  generic map 
    (
    JTAG_CHAIN     => 3               -- Value for USER command. Possible values: (1,2,3 or 4).
    )
  port map 
    (
    CAPTURE        => bscan_capture_c_a, -- 1-bit output: CAPTURE output from TAP controller.
    DRCK           => open,            -- 1-bit output: Gated TCK output. When SEL is asserted, DRCK toggles when CAPTURE or SHIFT are asserted.
    RESET          => open,            -- 1-bit output: Reset output for TAP controller.
    RUNTEST        => open,            -- 1-bit output: Output asserted when TAP controller is in Run Test/Idle state.
    SEL            => bscan_sel_c_a,     -- 1-bit output: USER active output.
    SHIFT          => bscan_shift_c_a,   -- 1-bit output: SHIFT output from TAP controller.
    TCK            => bscan_tck_c_a,     -- 1-bit output: Scan Clock output. Fabric connection to TAP Clock pin.
    TDI            => open,            -- 1-bit output: TDI output from TAP controller.
    TMS            => open,            -- 1-bit output: Test Mode Select input. Fabric connection to TAP.
    UPDATE         => bscan_update_c_a,  -- 1-bit output: UPDATE output from TAP controller
    TDO            => bscan_tdo      -- 1-bit input:  Data input for USER function.
    );
---------------------------------------------------------------------------------------
BSCAN_B: BSCANE2
  generic map 
    (
    JTAG_CHAIN     => 4               -- Value for USER command. Possible values: (1,2,3 or 4).
    )
  port map 
    (
    CAPTURE        => bscan_capture_c_b, -- 1-bit output: CAPTURE output from TAP controller.
    DRCK           => open,            -- 1-bit output: Gated TCK output. When SEL is asserted, DRCK toggles when CAPTURE or SHIFT are asserted.
    RESET          => open,            -- 1-bit output: Reset output for TAP controller.
    RUNTEST        => open,            -- 1-bit output: Output asserted when TAP controller is in Run Test/Idle state.
    SEL            => bscan_sel_c_b,     -- 1-bit output: USER active output.
    SHIFT          => bscan_shift_c_b,   -- 1-bit output: SHIFT output from TAP controller.
    TCK            => bscan_tck_c_b,     -- 1-bit output: Scan Clock output. Fabric connection to TAP Clock pin.
    TDI            => bscan_tdi,            -- 1-bit output: TDI output from TAP controller.
    TMS            => open,            -- 1-bit output: Test Mode Select input. Fabric connection to TAP.
    UPDATE         => bscan_update_c_b,  -- 1-bit output: UPDATE output from TAP controller
    TDO            => open      -- 1-bit input:  Data input for USER function.
    );

bscan_tdo                    <= status_reg(0);



bscan_p:
  process(bscan_tck_c_a)
    begin
       if bscan_tck_c_a'event and bscan_tck_c_a = '1' then
        
        -- The Shift Register Output is loaded when the BSCAN CAPTURE signal is asserted. 
        if bscan_capture_c_a = '1' then           
                status_reg(0)      <= result(0);
                status_reg(1)      <= result(1);
                status_reg(2)      <= result(2);             
                status_reg(31 downto 3)<= (others=>'0');

        end if;
        
        
        -- The Status Register is shifted out to the TDO pin 
        if bscan_sel_c_a = '1' and bscan_shift_c_a = '1' then
            for i in 0 to 30 loop
                status_reg(i)    <= status_reg(i+1);
            end loop;
        end if;
        
        end if;

    end process bscan_p;

read_logic:
  process(bscan_tck_c_b)
    begin

      if bscan_tck_c_b'event and bscan_tck_c_b = '1' then
        -- The data available on the TDI pin is shifted in to the Shift Register Input. 
        if bscan_sel_c_b = '1' and bscan_shift_c_b = '1' then
         
          shift_reg_in(31)    <= bscan_tdi;
          for i in 0 to 30 loop
            shift_reg_in(i)  <= shift_reg_in(i+1);
          end loop;
        end if;
        if bscan_update_c_b = '1' then
            read_reg<=shift_reg_in;
        end if;         
      end if;
  
    end process read_logic;
	

end Behavioral;
